from pact_test.utils.logger import *
from pact_test.constants import FAILED


def is_subset(expected, actual):
    actual_items = actual.items() if actual else {}
    expected_items = expected.items() if expected else {}

    stripped_actual_items = map(strip_whitespaces_after_commas, actual_items)
    stripped_expected_items = map(strip_whitespaces_after_commas, expected_items)

    return all(item in stripped_actual_items for item in stripped_expected_items)


def strip_whitespaces_after_commas(t):
    k = t[0]
    v = t[1].replace(', ', ',') if type(t[1]) is str else t[1]

    return k, v


def build_error_message(section, expected, actual):
    return {
        'actual': actual,
        'status': FAILED,
        'expected': expected,
        'message': section.capitalize() + ' is incorrect'
    }


def is_string(text):
    try:
        return True if (type(text) is str or type(text) is unicode) else False
    except NameError:
        return True if type(text) is str else False


def match_dicts_all_keys_and_values(d1, d2):
    d1_keys = d1.keys()
    d2_keys = d2.keys()

    delete_extra_keys(d1, d2)

    all_keys = set(d2_keys).issubset(set(d1_keys))
    all_values = match_dicts_all_values(d1, d2)

    return all_keys and all_values


def match_dicts_all_values(d1, d2):
    all_values = True
    for (k1, v1), (k2, v2) in zip(sorted(d2.items()), sorted(d1.items())):
        all_values = all_values and (v1 == v2)
    return all_values


def delete_extra_keys(d1, d2):
    extra_keys = list(set(d2.keys()) - set(d1.keys()))
    for extra_key in extra_keys:
        d2.pop(extra_key, None)
    for (k1, v1), (k2, v2) in zip(sorted(d1.items()), sorted(d2.items())):
        if type(v1) is dict and type(v2) is dict:
            delete_extra_keys(v1, v2)



class PactResponse(object):
    body = None
    headers = []
    status = 500

    def __init__(self, status=None, body=None, headers=None):
        self.body = body or self.body
        self.status = status or self.status
        self.headers = headers or self.headers



import os
import imp
import inspect
from pact_test.either import *
from pact_test.constants import *
from pact_test.utils.logger import *
from pact_test.utils.pact_utils import get_pact
from pact_test.utils.pact_helper_utils import load_pact_helper
from pact_test.runners.service_consumers.state_test import verify_state


class ServiceConsumerTestSuiteRunner(object):
    pact_helper = None

    def __init__(self, config):
        self.config = config

    def verify(self):
        print('')
        debug('Verify consumers: START')
        pact_helper = load_pact_helper(self.config.consumer_tests_path)
        if type(pact_helper) is Right:
            self.pact_helper = pact_helper.value
            tests = self.collect_tests()
            if type(tests) is Right:
                debug(str(len(tests.value)) + ' test(s) found.')
                debug('Execute Pact Helper setup: START')
                self.pact_helper.setup()
                debug('Execute Pact Helper setup: DONE')
                test_results = Right(list(map(self.verify_test, tests.value)))
                debug('Execute Pact Helper tear down: START')
                self.pact_helper.tear_down()
                debug('Execute Pact Helper tear down: DONE')
                debug('Verify consumers: DONE')
                return test_results
            error('Verify consumers: EXIT WITH ERRORS:')
            error(tests.value)
            return tests
        error('Verify consumers: EXIT WITH ERRORS:')
        error(pact_helper.value)
        return pact_helper

    def verify_test(self, test):
        validity_check = test.is_valid()
        if type(validity_check) is Right:
            pact = get_pact(test.pact_uri)
            if type(pact) is Right:
                interactions = pact.value.get('interactions', {})
                debug(str(len(interactions)) + ' interaction(s) found')
                test_results = [verify_state(i, self.pact_helper, test) for i in interactions]
                return Right({'test': test.__class__.__name__, 'results': test_results})
            error(pact.value)
            return pact
        error(validity_check.value)
        return validity_check

    def collect_tests(self):
        root = self.config.consumer_tests_path
        files = list(filter(filter_rule, self.all_files()))
        files = list(map(lambda f: os.path.join(root, f), files))
        tests = []
        for idx, filename in enumerate(files):
            test = imp.load_source('test' + str(idx), filename)
            for name, obj in inspect.getmembers(test):
                if inspect.isclass(obj) and len(inspect.getmro(obj)) > 2:
                    test_parent = inspect.getmro(obj)[1].__name__
                    if test_parent == TEST_PARENT:
                        tests.append(obj())

        if not files:
            return Left(MISSING_TESTS)
        return Right(tests)

    def all_files(self):
        return os.listdir(self.config.consumer_tests_path)


def filter_rule(filename):
    return (filename != '__init__.py' and
            filename.endswith('.py') and
            filename.endswith('pact_helper.py') is False)
